/*
 * HostUFO
 *
 * Adds UFO messages to output
 *
 * by Roger Burton West <roger@firedrake.demon.co.uk>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tkhost.h>
#include <tkiocomm.h>

const char ver[]="0.1";

void dohelp();
void send_message(int to,XString message);

/* invocation:
hostufo dir
*/

int main(int argc, char *argv[])
{
  UFO_DATA *ufopad;
  PLANETDATA *planpad;
  SHIPDATA *shippad;
  FILE *log,*own;
  bool send[12];

  extraplan=true;

  if (argc!=2) { // check argument count only
    fprintf(stderr,"Error in invocation!\n");
    dohelp();
    exit(1);
  }
  set_game_dir(argv[1]);

  own=mkopen("hostufo.hst",false,'r');
  if (!own) {
    for (int t=1;t<=11;t++) {
      send[t]=true;
    }
  } else {
    for (int t=1;t<=11;t++) {
      if (readUB1(own)==1) {
        send[t]=true;
      } else {
        send[t]=false;
      }
    }
  }
  fclose(own);

  log=mkopen("hostufo.log",false,'o');
  fprintf(log,"HostUFO v%s running in directory %s\n",ver,argv[1]);

  ufopad=read_all_ufo_data();
  if (!ufopad) {
    fprintf(log,"Couldn't open ufo.hst\n");
    return 1;
  }
  planpad=read_all_planetdata();
  if (!planpad) {
    fprintf(log,"Couldn't open pdata.hst\n");
    return(1);
  }
  shippad=read_all_shipdata();
  if (!shippad) {
    fprintf(log,"Couldn't open ships.hst\n");
    return(1);
  }

  for(int t=1;t<=500;t++) {
    int r;
    r=planpad[t].wOwner;
    if (planpad[t].sCode.Lower()=="con") {
      XString mess;
      mess="(-a0000)<<< HostUFO config message >>>\r\rUFO messages are ";
      if (send[r]) {
        mess|="ON";
      } else {
        mess|="OFF";
      }
      mess|=".\r\rSet planetary codes:\r";
      mess|="UFO - to receive messages\r";
      mess|="ufo - to stop messages\r";
      send_message(r,mess);
    }
    if (planpad[t].sCode=="UFO") {
      send[r]=true;
    }
    if (planpad[t].sCode=="ufo") {
      send[r]=false;
    }
  }

  own=mkopen("hostufo.hst",false,'o');
  if (own) {
    for (int t=1;t<=11;t++) {
      writeUB1(own,send[t]?1:0);
    }
    fclose(own);
  }

  for (int u=1;u<=100;u++) {
    if (ufopad[u].wMarker!=0) {
      bool racespot[12];
      bool any=false;
      int howmany=0;
      UB4 srange,prange;
      int r;
      for (int rn=1;rn<12;rn++) {
        racespot[rn]=false;
      }
      prange=ufopad[u].wRangeP;
      prange*=prange;
      for (int p=1;(p<501 && howmany<11);p++) {
        if (planpad[p].wOwner!=0) {
          if ((ufopad[u].pos*planpad[p].pos)<=prange) {
            r=planpad[p].wOwner;
            if (!racespot[r]) {
              howmany++;
            }
            any=true;
            racespot[r]=true;
            fprintf(log,"UFO %d spotted by race %d, planet %d\n",u,r,p);
          }
        }
      }
      if (howmany<11) {
        srange=ufopad[u].wRangeS;
        srange*=srange;
        for (int s=1;(s<501 && howmany<11);s++) {
          if (shippad[s].wRace!=0) {
            if ((ufopad[u].pos*shippad[s].pos)<=srange) {
              r=shippad[s].wRace;
              if (!racespot[r]) {
                howmany++;
              }
              any=true;
              racespot[r]=true;
              fprintf(log,"UFO %d spotted by race %d, ship %d\n",u,r,s);
            }
          }
        }
      }
      if (any) {
        XString mess;
        char spad[20];
        mess="(-a0000)<<< UFO DETECTED >>>\r\r";
        mess|="Name: " | ufopad[u].sNm | "\r";
        mess|="Info 1: " | ufopad[u].Info1 | "\r";
        mess|="Info 2: " | ufopad[u].Info2 | "\r\r";
        mess|="X: ";
        sprintf(spad,"%d",ufopad[u].pos.x);
        mess|=spad;
        mess|="\r";
        mess|="Y: ";
        sprintf(spad,"%d",ufopad[u].pos.y);
        mess|=spad;
        mess|="\r";
        mess|="Warp: ";
        sprintf(spad,"%d",ufopad[u].wWarp);
        mess|=spad;
        mess|="\r";
        mess|="Heading: ";
        sprintf(spad,"%d",ufopad[u].wHeading);
        mess|=spad;
        mess|="\r";
        mess|="Radius: ";
        sprintf(spad,"%d",ufopad[u].wEdge);
        mess|=spad;
        mess|="\r\r";
        mess|="Ship vis range: ";
        sprintf(spad,"%d",ufopad[u].wRangeS);
        mess|="\r";
        mess|="Planet vis range: ";
        sprintf(spad,"%d",ufopad[u].wRangeP);
        mess|="\r";
        mess|="Marker colour: ";
        sprintf(spad,"%d",ufopad[u].wMarker);
        mess|="\r";
        mess|="Parent ID: ";
        sprintf(spad,"%d",ufopad[u].wParent);
        mess|=spad;
        mess|="\r";
        for (int rn=1;rn<12;rn++) {
          if (racespot[rn] && send[rn]) {
            fprintf(log,"UFO %d message to race %d\n",u,rn);
            send_message(rn,mess);
          }
        }
      }
    }
  }

  delete[] planpad;
  delete[] shippad;
  delete[] ufopad;

  fprintf(log,"UFO shutting down.\n\n");
  fclose(log);

  return 0;
}

void dohelp()
{
  fprintf(stderr,"HOSTUFO vv%s - by Roger Burton West <roger@firedrake.demon.co.uk>\n",ver);
  fprintf(stderr,"\n");
  fprintf(stderr,"Invocation:\n");
  fprintf(stderr,"  HostUFO <gamedir>\n");
  fprintf(stderr,"\n");
  fprintf(stderr,"  gamedir: the game directory\n");
  return;
}

void send_message(int to,XString message)
{
  OUTMESS temp;
  temp.iTo=to;
  temp.sMessage=message;
  write_outmess(0,&temp);
}
